@@ -6,14 +6,11 @@
 #
 # SPDX-License-Identifier: GPL-2.0-or-later
 
-from __future__ import division, print_function
-
-import argparse
 import binascii
 import re
 import sys
 
-from bitstring import BitArray, BitString
+from bitstring import BitArray, BitString, CreationError
 
 import esptool
 
@@ -30,7 +27,7 @@ class CheckArgValue(object):
             if efuse.efuse_type.startswith("bool"):
                 new_value = 1 if new_value is None else int(new_value, 0)
                 if new_value != 1:
-                    raise argparse.ArgumentTypeError(
+                    raise esptool.FatalError(
                         "New value is not accepted for efuse '{}' "
                         "(will always burn 0->1), given value={}".format(
                             efuse.name, new_value
@@ -50,32 +47,32 @@ class CheckArgValue(object):
                         new_value = int(new_value, 0)
                 else:
                     if new_value is None:
-                        raise argparse.ArgumentTypeError(
+                        raise esptool.FatalError(
                             "New value required for efuse '{}' (given None)".format(
                                 efuse.name
                             )
                         )
                     new_value = int(new_value, 0)
                     if new_value == 0:
-                        raise argparse.ArgumentTypeError(
+                        raise esptool.FatalError(
                             "New value should not be 0 for '{}' "
                             "(given value= {})".format(efuse.name, new_value)
                         )
             elif efuse.efuse_type.startswith("bytes"):
                 if new_value is None:
-                    raise argparse.ArgumentTypeError(
+                    raise esptool.FatalError(
                         "New value required for efuse '{}' "
                         "(given None)".format(efuse.name)
                     )
                 if len(new_value) * 8 != efuse.bitarray.len:
-                    raise argparse.ArgumentTypeError(
+                    raise esptool.FatalError(
                         "The length of efuse '{}' ({} bits) "
                         "(given len of the new value= {} bits)".format(
                             efuse.name, efuse.bitarray.len, len(new_value) * 8
                         )
                     )
             else:
-                raise argparse.ArgumentTypeError(
+                raise esptool.FatalError(
                     "The '{}' type for the '{}' efuse is not supported yet.".format(
                         efuse.efuse_type, efuse.name
                     )
@@ -461,6 +458,13 @@ class EspEfusesBase(object):
     def efuse_controller_setup(self):
         pass
 
+    def reconnect_chip(self, esp):
+        print("Re-connecting...")
+        baudrate = esp._port.baudrate
+        port = esp._port.port
+        esp._port.close()
+        return esptool.cmds.detect_chip(port, baudrate)
+
     def get_index_block_by_name(self, name):
         for block in self.blocks:
             if block.name == name or name in block.alias:
@@ -600,7 +604,15 @@ class EfuseFieldBase(EfuseProtectBase):
                 # *[x] - means a byte.
                 return BitArray(bytes=new_value[::-1], length=len(new_value) * 8)
             else:
-                return BitArray(self.efuse_type + "={}".format(new_value))
+                try:
+                    return BitArray(self.efuse_type + "={}".format(new_value))
+                except CreationError as err:
+                    print(
+                        "New value '{}' is not suitable for {} ({})".format(
+                            new_value, self.name, self.efuse_type
+                        )
+                    )
+                    raise esptool.FatalError(err)
 
     def check_new_value(self, bitarray_new_value):
         bitarray_old_value = self.get_bitstring() | self.get_bitstring(from_read=False)
@@ -648,6 +660,9 @@ class EfuseFieldBase(EfuseProtectBase):
         self.save_to_block(bitarray_field)
 
     def update(self, bit_array_block):
+        if self.word is None or self.pos is None:
+            self.bitarray.overwrite(self.convert_to_bitstring(self.get()), pos=0)
+            return
         field_len = self.bitarray.len
         bit_array_block.pos = bit_array_block.length - (
             self.word * 32 + self.pos + field_len
