@@ -3,10 +3,10 @@
 #
 # SPDX-License-Identifier: GPL-2.0-or-later
 
-from __future__ import division, print_function
-
+import base64
 import hashlib
 import itertools
+import json
 import os
 import re
 import string
@@ -73,6 +73,15 @@ ERASE_WRITE_TIMEOUT_PER_MB = 40  # timeout (per megabyte) for erasing and writin
 MEM_END_ROM_TIMEOUT = 0.05  # short timeout for ESP_MEM_END, as it may never respond
 DEFAULT_SERIAL_WRITE_TIMEOUT = 10  # timeout for serial port write
 DEFAULT_CONNECT_ATTEMPTS = 7  # default number of times to try connection
+WRITE_BLOCK_ATTEMPTS = 3  # number of times to try writing a data block
+
+STUBS_DIR = os.path.join(os.path.dirname(__file__), "./targets/stub_flasher/")
+
+
+def get_stub_json_path(chip_name):
+    chip_name = re.sub(r"[-()]", "", chip_name.lower())
+    chip_name = chip_name.replace("esp", "")
+    return STUBS_DIR + "stub_flasher_" + chip_name + ".json"
 
 
 def timeout_per_mb(seconds_per_mb, size_bytes):
@@ -123,6 +132,23 @@ def esp32s3_or_newer_function_only(func):
     )
 
 
+class StubFlasher:
+    def __init__(self, json_path):
+        with open(json_path) as json_file:
+            stub = json.load(json_file)
+
+        self.text = base64.b64decode(stub["text"])
+        self.text_start = stub["text_start"]
+        self.entry = stub["entry"]
+
+        try:
+            self.data = base64.b64decode(stub["data"])
+            self.data_start = stub["data_start"]
+        except KeyError:
+            self.data = None
+            self.data_start = None
+
+
 class ESPLoader(object):
     """Base class providing access to ESP ROM & software stub bootloaders.
     Subclasses provide ESP8266 & ESP32 Family specific functionality.
@@ -243,7 +269,10 @@ class ESPLoader(object):
         self.stub_is_disabled = False
 
         if isinstance(port, str):
-            self._port = serial.serial_for_url(port)
+            try:
+                self._port = serial.serial_for_url(port)
+            except serial.serialutil.SerialException:
+                raise FatalError(f"Could not open {port}, the port doesn't exist")
         else:
             self._port = port
         self._slip_reader = slip_reader(self._port, self.trace)
@@ -697,12 +726,12 @@ class ESPLoader(object):
         """Start downloading an application image to RAM"""
         # check we're not going to overwrite a running stub with this data
         if self.IS_STUB:
-            stub = self.STUB_CODE
+            stub = StubFlasher(get_stub_json_path(self.CHIP_NAME))
             load_start = offset
             load_end = offset + size
             for (start, end) in [
-                (stub["data_start"], stub["data_start"] + len(stub["data"])),
-                (stub["text_start"], stub["text_start"] + len(stub["text"])),
+                (stub.data_start, stub.data_start + len(stub.data)),
+                (stub.text_start, stub.text_start + len(stub.text)),
             ]:
                 if load_start < end and load_end > start:
                     raise FatalError(
@@ -775,29 +804,51 @@ class ESPLoader(object):
         return num_blocks
 
     def flash_block(self, data, seq, timeout=DEFAULT_TIMEOUT):
-        """Write block to flash"""
-        self.check_command(
-            "write to target Flash after seq %d" % seq,
-            self.ESP_FLASH_DATA,
-            struct.pack("<IIII", len(data), seq, 0, 0) + data,
-            self.checksum(data),
-            timeout=timeout,
-        )
+        """Write block to flash, retry if fail"""
+        for attempts_left in range(WRITE_BLOCK_ATTEMPTS - 1, -1, -1):
+            try:
+                self.check_command(
+                    "write to target Flash after seq %d" % seq,
+                    self.ESP_FLASH_DATA,
+                    struct.pack("<IIII", len(data), seq, 0, 0) + data,
+                    self.checksum(data),
+                    timeout=timeout,
+                )
+                break
+            except FatalError:
+                if attempts_left:
+                    self.trace(
+                        "Block write failed, "
+                        f"retrying with {attempts_left} attempts left"
+                    )
+                else:
+                    raise
 
     def flash_encrypt_block(self, data, seq, timeout=DEFAULT_TIMEOUT):
-        """Encrypt before writing to flash"""
+        """Encrypt, write block to flash, retry if fail"""
         if self.SUPPORTS_ENCRYPTED_FLASH and not self.IS_STUB:
             # ROM support performs the encrypted writes via the normal write command,
             # triggered by flash_begin(begin_rom_encrypted=True)
             return self.flash_block(data, seq, timeout)
 
-        self.check_command(
-            "Write encrypted to target Flash after seq %d" % seq,
-            self.ESP_FLASH_ENCRYPT_DATA,
-            struct.pack("<IIII", len(data), seq, 0, 0) + data,
-            self.checksum(data),
-            timeout=timeout,
-        )
+        for attempts_left in range(WRITE_BLOCK_ATTEMPTS - 1, -1, -1):
+            try:
+                self.check_command(
+                    "Write encrypted to target Flash after seq %d" % seq,
+                    self.ESP_FLASH_ENCRYPT_DATA,
+                    struct.pack("<IIII", len(data), seq, 0, 0) + data,
+                    self.checksum(data),
+                    timeout=timeout,
+                )
+                break
+            except FatalError:
+                if attempts_left:
+                    self.trace(
+                        "Encrypted block write failed, "
+                        f"retrying with {attempts_left} attempts left"
+                    )
+                else:
+                    raise
 
     def flash_finish(self, reboot=False):
         """Leave flash mode and run/reboot"""
@@ -849,6 +900,9 @@ class ESPLoader(object):
 
     @classmethod
     def parse_flash_freq_arg(cls, arg):
+        if arg is None:
+            # The encoding of the default flash frequency in FLASH_FREQUENCY is always 0
+            return 0
         try:
             return cls.FLASH_FREQUENCY[arg]
         except KeyError:
@@ -860,7 +914,7 @@ class ESPLoader(object):
 
     def run_stub(self, stub=None):
         if stub is None:
-            stub = self.STUB_CODE
+            stub = StubFlasher(get_stub_json_path(self.CHIP_NAME))
 
         if self.sync_stub_detected:
             print("Stub is already running. No upload is necessary.")
@@ -868,18 +922,18 @@ class ESPLoader(object):
 
         # Upload
         print("Uploading stub...")
-        for field in ["text", "data"]:
-            if field in stub:
-                offs = stub[field + "_start"]
-                length = len(stub[field])
+        for field in [stub.text, stub.data]:
+            if field is not None:
+                offs = stub.text_start if field == stub.text else stub.data_start
+                length = len(field)
                 blocks = (length + self.ESP_RAM_BLOCK - 1) // self.ESP_RAM_BLOCK
                 self.mem_begin(length, blocks, self.ESP_RAM_BLOCK, offs)
                 for seq in range(blocks):
                     from_offs = seq * self.ESP_RAM_BLOCK
                     to_offs = from_offs + self.ESP_RAM_BLOCK
-                    self.mem_block(stub[field][from_offs:to_offs], seq)
+                    self.mem_block(field[from_offs:to_offs], seq)
         print("Running stub...")
-        self.mem_finish(stub["entry"])
+        self.mem_finish(stub.entry)
 
         p = self.read()
         if p != b"OHAI":
@@ -931,14 +985,25 @@ class ESPLoader(object):
 
     @stub_and_esp32_function_only
     def flash_defl_block(self, data, seq, timeout=DEFAULT_TIMEOUT):
-        """Write block to flash, send compressed"""
-        self.check_command(
-            "write compressed data to flash after seq %d" % seq,
-            self.ESP_FLASH_DEFL_DATA,
-            struct.pack("<IIII", len(data), seq, 0, 0) + data,
-            self.checksum(data),
-            timeout=timeout,
-        )
+        """Write block to flash, send compressed, retry if fail"""
+        for attempts_left in range(WRITE_BLOCK_ATTEMPTS - 1, -1, -1):
+            try:
+                self.check_command(
+                    "write compressed data to flash after seq %d" % seq,
+                    self.ESP_FLASH_DEFL_DATA,
+                    struct.pack("<IIII", len(data), seq, 0, 0) + data,
+                    self.checksum(data),
+                    timeout=timeout,
+                )
+                break
+            except FatalError:
+                if attempts_left:
+                    self.trace(
+                        "Compressed block write failed, "
+                        f"retrying with {attempts_left} attempts left"
+                    )
+                else:
+                    raise
 
     @stub_and_esp32_function_only
     def flash_defl_finish(self, reboot=False):
@@ -1366,6 +1431,38 @@ def slip_reader(port, trace_function):
     Designed to avoid too many calls to serial.read(1), which can bog
     down on slow systems.
     """
+
+    def detect_panic_handler(input):
+        """
+        Checks the input bytes for panic handler messages.
+        Raises a FatalError if Guru Meditation or Fatal Exception is found, as both
+        of these are used between different ROM versions.
+        Tries to also parse the error cause (e.g. IllegalInstruction).
+        """
+
+        guru_meditation = (
+            rb"G?uru Meditation Error: (?:Core \d panic'ed \(([a-zA-Z]*)\))?"
+        )
+        fatal_exception = rb"F?atal exception \(\d+\): (?:([a-zA-Z]*)?.*epc)?"
+
+        # Search either for Guru Meditation or Fatal Exception
+        data = re.search(
+            rb"".join([rb"(?:", guru_meditation, rb"|", fatal_exception, rb")"]),
+            input,
+            re.DOTALL,
+        )
+        if data is not None:
+            msg = "Guru Meditation Error detected {}".format(
+                " ".join(
+                    [
+                        "({})".format(i.decode("utf-8"))
+                        for i in [data.group(1), data.group(2)]
+                        if i is not None
+                    ]
+                )
+            )
+            raise FatalError(msg)
+
     partial_packet = None
     in_escape = False
     successful_slip = False
@@ -1393,10 +1490,12 @@ def slip_reader(port, trace_function):
                     partial_packet = b""
                 else:
                     trace_function("Read invalid data: %s", HexFormatter(read_bytes))
+                    remaining_data = port.read(port.inWaiting())
                     trace_function(
                         "Remaining data in serial buffer: %s",
-                        HexFormatter(port.read(port.inWaiting())),
+                        HexFormatter(remaining_data),
                     )
+                    detect_panic_handler(read_bytes + remaining_data)
                     raise FatalError(
                         "Invalid head of packet (0x%s): "
                         "Possible serial noise or corruption." % hexify(b)
@@ -1409,10 +1508,12 @@ def slip_reader(port, trace_function):
                     partial_packet += b"\xdb"
                 else:
                     trace_function("Read invalid data: %s", HexFormatter(read_bytes))
+                    remaining_data = port.read(port.inWaiting())
                     trace_function(
                         "Remaining data in serial buffer: %s",
-                        HexFormatter(port.read(port.inWaiting())),
+                        HexFormatter(remaining_data),
                     )
+                    detect_panic_handler(read_bytes + remaining_data)
                     raise FatalError("Invalid SLIP escape (0xdb, 0x%s)" % (hexify(b)))
             elif b == b"\xdb":  # start of escape sequence
                 in_escape = True
